SystemOrganization addCategory: #FuelPackageLoaderTests!

TestCase subclass: #FLPackageLoaderTest
	uses: FLTSerializationTest
	instanceVariableNames: 'streams classFactory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoaderTests'!

FLPackageLoaderTest class
	uses: FLTSerializationTest classTrait
	instanceVariableNames: ''!

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
classFactory
	^ classFactory! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
classFactory: anObject
	classFactory := anObject! !

!FLPackageLoaderTest methodsFor: 'running' stamp: 'MartinDias 8/21/2011 21:11'!
storeRemoveAndLoadStubPackage

	| aStream package |
	aStream := self writeStream.
	FLPackageStore new storeOn: aStream packageNamed: classFactory packageName.
	
	classFactory cleanUp.
	self stubPackageInfo extensionMethods do: [ :method | method methodClass removeSelector: method selector ].
	self assert: self stubPackageInfo classes isEmpty.
	self assert: self stubPackageInfo extensionMethods isEmpty.

	package := FLPackageLoader new loadFrom: aStream reset.
	classFactory createdClasses: package classes! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
streams
	^ streams! !

!FLPackageLoaderTest methodsFor: 'accessing' stamp: 'MartinDias 3/26/2011 03:23'!
streams: anObject
	streams := anObject! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 3/26/2011 04:12'!
stubPackageInfo
	^ PackageInfo named: classFactory packageName! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 02:57'!
testClassInitialization
	"It should send #initialize to each class."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class 
		compileSilently: 'initialize  ClassVariable := 1';
		compileSilently: 'isInitialized  ^ClassVariable isNil not'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.
	
	self assert: materializedClass isInitialized.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 07:14'!
testClassRegistration
	"The class should be included in Smalltalk system dictionary."

	| aClass name materializedClass |
	aClass := classFactory newClass.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.

	materializedClass := 
		Smalltalk 
			at: name 
			ifAbsent: [ self signalFailure: 'The class should be included in Smalltalk system dictionary.' ].
			
	self assert: classFactory defaultCategory equals: (Smalltalk organization categoryOfElement: materializedClass name).
! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 20:49'!
testIgnoreClassSideVariable
	"It should not serialize class side variable values."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass class
		addInstVarNamed: 'classSideVariable';  
		compileSilently: 'initializeClassSideVariable  classSideVariable := 1';
		compileSilently: 'classSideVariable  ^classSideVariable'.
	aClass initializeClassSideVariable.
	self deny: aClass classSideVariable isNil.
	
	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classSideVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:13'!
testIgnoreClassVariableWithClassSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:12'!
testIgnoreClassVariableWithInstanceSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass new initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass new classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:14'!
testIgnoreForeignClassSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass class
		compileSilently: 'foreign' classified: '*foreigns';
		compileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass class localSelectors.	
	self assert: (materializedClass class includesSelector: #core).
	self deny: (materializedClass class includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass class organization categories.
	self assert: #(core) equals: materializedClass class organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/22/2011 04:14'!
testIgnoreForeignInstanceSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass
		compileSilently: 'foreign' classified: '*foreigns';
		compileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass localSelectors.	
	self assert: (materializedClass includesSelector: #core).
	self deny: (materializedClass includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass organization categories.
	self assert: #(core) equals: materializedClass organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/21/2011 15:56'!
testIgnoreSuperClassSideVariable
	"It should not serialize class side variable values even if they come from super."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newClass.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.
	
	aSuperclass class
		addInstVarNamed: 'classSideVariable'.
	aClass class
		compileSilently: 'initializeClassSideVariable  classSideVariable := 1';
		compileSilently: 'classSideVariable  ^classSideVariable'.
		
	aClass initializeClassSideVariable.
	self deny: aClass classSideVariable isNil.
	
	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classSideVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/21/2011 16:04'!
testIgnoreSuperClassVariableWithClassSideMethod
	"It should not serialize class variable values even if it comes from super."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.

	aClass class
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2011 01:15'!
testIgnoreSuperClassVariableWithInstanceSideMethod
	"It should not serialize class variable values."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.
	aClass
		compileSilently: 'initializeClassVariable  ClassVariable := 1';
		compileSilently: 'classVariable  ^ClassVariable'.
	aClass new initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass new classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2011 15:43'!
testPackageWithMethodExtension
	"It should serialize package's extension methods."

	| classWithExtension categoryForMethodExtension |
	categoryForMethodExtension := self stubPackageInfo methodCategoryPrefix.
	classWithExtension := FLPackageLoaderDummy.
	classWithExtension compileSilently: 'fortyTwo ^42' classified: categoryForMethodExtension. 
	
	self storeRemoveAndLoadStubPackage.
	
	self assert: (classWithExtension includesSelector: #fortyTwo).
	self assert: categoryForMethodExtension equals: (classWithExtension whichCategoryIncludesSelector: #fortyTwo).
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName)! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2011 15:43'!
testPackageWithMethodExtensionWithClassSideVariable
	"Tests that the extension method is corretly rebinded on load."

	| categoryForMethodExtension |
	categoryForMethodExtension := self stubPackageInfo methodCategoryPrefix.
	FLPackageLoaderDummy class 
		compileSilently: 'initializeFortyTwo  ClassVariable := 42' 
		classified: categoryForMethodExtension. 

	"After next lines, the class method points to an old binding. (This is not the ideal way of testing because it is breaking consistency of the class.)"
	FLPackageLoaderDummy classPool removeKey: #ClassVariable.
	FLPackageLoaderDummy classPool at: #ClassVariable put: nil.

	self storeRemoveAndLoadStubPackage.

	"Bindings in the class method should be recalculated on load."	
	self shouldnt: [ FLPackageLoaderDummy initializeFortyTwo ] raise: Error.
	self assert: 42 equals: (FLPackageLoaderDummy classPool at: #ClassVariable).

! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/21/2011 22:04'!
testPackageWithTwoCategories
	"It should serialize multiple categories by package."

	| aStream classNames |
	classFactory newClassInCategory: 'One'.
	classFactory newClassInCategory: 'Two'.
	classNames := classFactory createdClassNames.

	self storeRemoveAndLoadStubPackage.
	
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName) description: 'The package should be present in PackageOrganizer default.'.! !

Object subclass: #FLPackageLoaderDummy
	instanceVariableNames: ''
	classVariableNames: 'ClassVariable'
	poolDictionaries: ''
	category: 'FuelPackageLoaderTests'!

!FLPackageLoaderDummy classSide methodsFor: 'accessing' stamp: 'MartinDias 9/22/2011 15:12'!
classVariable

	^ClassVariable! !

!FLPackageLoaderDummy classSide methodsFor: 'accessing' stamp: 'MartinDias 9/22/2011 15:12'!
classVariable: anObject

	ClassVariable := anObject ! !
